/***************************************************************************
 *  file                 :  element.c                                      *
 *  copyright            : (C) 2001 by Dirk Nolting	                   *
 *  email                : nolting@uni-duesseldorf.de                      *
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "global.h"
#include "element.h"
#include <unistd.h>

element *elements=NULL;

FILE *open_file(char *filename)
{
  FILE *datenfile;
  char *pwd;

  pwd=getcwd(NULL,0);

  if( (chdir(DATADIR)) == -1 )
    {
      printf("Can't change to directory %s\n",DATADIR);
      return 0;
    }
 
  if(!(datenfile=fopen(filename,"r")))
    {
      printf("Can't open file: %s.\n",filename);
      return 0;
    }
 if( (chdir(pwd)) == -1 )
    {
      printf("Can't change to directory %s\n",pwd);
      return 0;
    }
 return datenfile;
}


void add_element(element *ce)
{
  element *cur;
  
  if(!(elements))
    {
      elements=ce;
      elements->previous=NULL;
      return;
    }

  cur=elements;
  while(cur->next)
    cur=cur->next;

  cur->next=ce;
  ce->previous=cur;
}

void add_isotope(isotope *ci,element *ce)
{
  isotope *cur;

  if(!(ce->isotopes))
    {
      ce->isotopes=ci;
      ci->previous=NULL;
      return;
    }

  cur=ce->isotopes;
  while(cur->next)
    cur=cur->next;
  cur->next=ci;
  ci->previous=cur;
  return;
}

int init_elements() 
{
  FILE* data;
  char linebuffer[MAX_LINE];
  int count;
  element *ce;
  isotope *ci;

  if(!(data=open_file(ELEMENTFILE)))
    return 0;

  fgets(linebuffer,MAX_LINE,data);  // read table headings
  fgets(linebuffer,MAX_LINE,data);
  fgets(linebuffer,MAX_LINE,data);  // get first barred line

  for( count = 0; count < NUM_ELEMENTS; count++ ) {
    ce = parse_element(data,linebuffer);
    add_element(ce);
    ci = parse_isotope(data,linebuffer);

    // don't add isotopes with 0 probability since they don't affect results
    // but increase computing time.
    if( ci->p > 0.0 ) {
      add_isotope(ci,ce);
    }

    while(fgetc(data) != '_') {  //if the first char isn't a bar then this is an isotope line
      ci = parse_isotope(data,linebuffer);
      add_isotope(ci,ce);
    }
    fgets(linebuffer,MAX_LINE,data);  // otherwise read the rest of the bar line
  }
  
  fclose(data);
  return 1;
}

element* parse_element(FILE* data, char* linebuffer) {
  char* token;
  int number;
  element* ce=(element*)malloc(sizeof(element));
  ce->isotopes = NULL;
  
  // read enough characters to get the element number and symbol
  fgets(linebuffer,9,data);

  token = (char*)strtok(linebuffer," ");
  number = atoi(token);
  token = (char*)strtok(NULL," ");
  ce->symbol = strdup(token);
  
  //printf("Element symbol: %s\tElement number: %d\n",ce->symbol,number);
  return ce;
}

isotope* parse_isotope(FILE* data, char* linebuffer) {
  char* token;
  int number;
  isotope* ci =(isotope *)malloc(sizeof(isotope));


  fgets(linebuffer,MAX_LINE,data);

  token = (char*)strtok(linebuffer," ");   //get the number in 3rd column
  number = atoi(token);

  token = (char*)strtok(NULL,"( ");  // get the mass
  ci->mass = atof(token);

  token = (char*)strtok(NULL,")# "); // get the number in paretheses and remove
                                     // the ')', a '#',and any trailing spaces

  token = (char*)strtok(NULL,"(");   //get the isotopic composition
  ci->p = ( atof(token) )/100;

  //printf("\tisotope mass: %f\tisotope percent: %f\n",ci->mass,ci->p);
  return ci;
}
